# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.utils.translation import ugettext as _
from django import forms
from django.forms.formsets import formset_factory
import os
from datetime import time
from project.view import FuncView
import settings

from fields import (SyslogAlertNameField,
                    SyslogAlertMessageField,
                    SyslogAlertHostnameField,
                    SyslogAlertFacilitiesField,
                    SyslogAlertPrioritiesField)

from project.fal.syslogsrv import FALSyslogSrv, FACILITIES, PRIORITIES

MAX_NUM = 30

class AlertForm(forms.Form):

    name = SyslogAlertNameField()
    facilities = SyslogAlertFacilitiesField()
    priorities = SyslogAlertPrioritiesField()
    hostname = SyslogAlertHostnameField()
    message = SyslogAlertMessageField()
    
    def __init__(self, *args, **kwargs):
        super(AlertForm, self).__init__(*args, **kwargs)

    def clean(self):
        data = self.cleaned_data
        return data


AlertFormset = formset_factory(AlertForm,
                               extra=0,
                               can_delete=True,
                               max_num=MAX_NUM)


class AlertView(FuncView):

    views = ['apply']

    def view(self, request):
        formset = AlertFormset(prefix='alert', initial=self.__initial())

        template = os.path.join(self.TEMPLATES, 'template.html')
        dictionary = {
            'formset': formset,
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }

    def view_apply(self, request):
        formset = AlertFormset(prefix='alert', data=request.POST)

        with self.processing(constname="ERR_SYSLOG_CHANGE") as p:
            if formset.is_valid():
                name_is_unique = True
                names = []
                alertlist = []
                for form in formset:
                    if form.is_valid():
                        if form.cleaned_data == {}:
                            # formsetを使うと追加分のformはempty_permitted=Trueになるため
                            continue
                        if form.cleaned_data["DELETE"]:
                            # 削除フラグがたっているformでもformset.cleaned_dataに入ってくるため
                            continue
                        name = form.cleaned_data["name"]
                        if name in names:  # 重複チェック
                            name_is_unique = False
                            break
                        names.append(name)
                        alertlist.append({
                            "name": form.cleaned_data["name"],
                            "hostname": form.cleaned_data["hostname"],
                            "message": form.cleaned_data["message"],
                            "facilities": form.cleaned_data["facilities"],
                            "priorities": form.cleaned_data["priorities"],
                        })
                if name_is_unique:
                    fal = FALSyslogSrv()
                    p.result(fal.apply(data={"alertlist": alertlist}))
                    self.set_message(fal.get_message())
                    formset = AlertFormset(prefix='alert', initial=self.__initial())
                else:
                    self.set_message(_(u"syslogsrv: Please input unique name."))

        template = os.path.join(self.TEMPLATES, 'template.html')
        dictionary = {
            'formset': formset,
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }

    def __initial(self):
        try:
            initials = []
            settings = FALSyslogSrv().load()["alertlist"]
            for setting in settings:
                initials.append({
                    "name": setting["name"],
                    "hostname": setting["hostname"],
                    "message": setting["message"],
                    "facilities": setting["facilities"],
                    "priorities": setting["priorities"],
                })
            return initials
        except:
            return []
