# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from project.fal.command import Command

from ionas.request import Entry, XmlNode, XmlConfNode
from fal_info import Results, FAL_SYSTEMLOG_PATH


class FALSystemlog(Command):

    COMMAND = FAL_SYSTEMLOG_PATH

    def load(self):

        def __dict(entry):
            d = {}
            try:
                systemlog_node = entry.get()
                d["accesslog"] = systemlog_node.get_node("accesslog").is_enabled()
            except:
                d["accesslog"] = None
            return d

        try:
            entry = self._load()
            return __dict(entry)
        except:
            return {}

    def apply(self, data=None, log=True, messages={}):
        __default_messages = {
            Results.OK: ["MSG_LOGEXT_CHANGE", {}],
            None: ["ERR_LOGEXT_CHANGE", {}],
        }
        __default_messages.update(messages)

        try:
            entry = self._apply(data=data)
            result = getattr(entry, "get_result", lambda: None)()
        except:
            result = None

        if result not in [Results.OK]:
            message = __default_messages.get(result, __default_messages[None])
            self._set_message(message[0], arglist=message[1], log=log)

        return result in [Results.OK]

    def _load(self):
        entry = Entry("load", """<systemlog/>""")
        return self.request(entry)

    def _apply(self, data=None):
        entry = Entry("apply", """<systemlog/>""")
        systemlog_node = entry.get()

        if data.get("accesslog", None) is not None:
            systemlog_node.append_child("accesslog")
            accesslog_node = systemlog_node.get_node("accesslog")
            accesslog_node.enable(data["accesslog"])

        return self.request(entry)


if __name__ == u"__main__":
    fal = FALSystemlog()
    print fal.load()["accesslog"]
    print fal.apply(data={"accesslog": not fal.load()["accesslog"]})
    print fal.load()["accesslog"]

